#!/bin/sh

## Testing the NAND flash: MT29F4G16 (x16)
## 
## Page	 = 1056 words = (1K + 32) words 
## Block = 64 x pages = (64K + 2K) words  =>  (0x0002.0000 + 0x0000.1000) bytes 
## Total =  4096 blocks = 262144 pages = 512 MBytes = 4.224Mbits
##		 = (0x2000.0000 + 0x0100.0000) bytes
##
## Layout:
##    mtd0  -------------- (barebox & kernel)
## 			0x0000.0000 - 0x0004.0000	//Empty (two first blocks)
##			0x0004.0000 - 0x0008.0000	//Barebox env
##			0x0008.0000 - 0x0010.0000	//Empty (up to 1MB position)
##			0x0010.0000 - 0x0090.0000	//Kernel (size of 8MB)
##			0x0090.0000 - 0x0400.0000 	//Empty (up to the end of /dev/mtd0)
##    mtd1 -------------- (filesystem)  
##			0x0400.0000 - 0x2000.0000	//Filesystem space, jffs2
## 
## dev:    size   	erasesize  name
## mtd0 : 04000000 	00020000 "Partition 1"
## mtd1 : 1c000000 	00020000 "Partition 2"
	  	
displayErr() {

if [ "$md5w" != "$md5r" ]; then
	echo "ERROR @ $testdev W:$md5w, R:$md5r"
	cmp -l /testing/at91bootstrap.bin /tmp/nand-at91 > /tmp/cmprw
	head -n 1 /tmp/cmprw
	tail -n 1 /tmp/cmprw
	hexdump -n 500 -v /tmp/nand-at91 
else 
	echo "OK @ $testnand R/W" 
fi  
}

flashAllSkip()
{
	echo "execute $1?"
	read -t 5 -p "Press any key to skip it (5s): " ret; 
}


printf "======: $0\n"


npages=8192
ipage=0
dfdev=/dev/mtd0
f2w=/tmp/w1k.bin
f2r=/tmp/r1k.bin
nOK=0
nKO=0
ddopt="bs=1k count=1"

flashAllSkip "flash_eraseall ${dfdev}";
if [ -z ${ret} ]; then
	flash_eraseall ${dfdev}
fi

printf "Testing NAND memory\n"
okTxt=""
while [ $ipage -lt $npages ]; do
	
	if [ $(expr $ipage % 256)  -eq 0 ]; then
    		printf "${okTxt}\n\nSector %02d:" $(expr $ipage / 256)
    		okTxt=""
	fi
	
	if [ $(expr $ipage % 32)  -eq 0 ]; then
    		printf "${okTxt}\nB%04d:" $(expr $ipage / 32)
    		okTxt=" OK"
	fi

	startaddr=$(expr $ipage \* 1056)
	

	dd $ddopt if=/dev/urandom of=${f2w} 		  2> /dev/null
#	dd $ddopt if=${f2w} of=${dfdev} seek=${startaddr} 2> /dev/null
#	dd $ddopt if=${dfdev} of=${f2r} skip=${startaddr} 2> /dev/null

#	md5w=`md5sum ${f2w} | cut -f1 -d" "`
#	md5r=`md5sum ${f2r} | cut -f1 -d" "`

	md5w="0215"
	md5r="0215"

	if [ "$md5w" != "$md5r" ]; then
		nKO=$(expr ${nKO} + 1)		
		echo "ERROR @ $startaddr"
	else
		nOK=$(expr ${nOK} + 1) 
		printf ".."
	fi

	ipage=$(expr $ipage + 1)         


done

echo ""
echo "Result DF testing: OK=${nOK}/${npages}"

